/**
 * scoring.js
 * Copyright (C) 2010-2011 Tommi Rautava
 * 
 * This file is part of Popomungo.
 *
 * Popomungo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Popomungo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

org.mozdev.popomungo.classes.MenuHandler = function MenuHandler() {
	this._logger = org.mozdev.popomungo.logger;
};

org.mozdev.popomungo.classes.MenuHandler.prototype = {
	
	handleEvent: function handleEvent(aEvent) {
		this._logger.debug("Event="+ aEvent.type);
		
		switch (aEvent.type) {
			case "popupshowing": 
				this.contextMenuPopupShowing(aEvent);
				break;

			default: 
				this._logger.logError("Unexpected event: "+ aEvent.type);
		}
	},
	
	
	contextMenuPopupShowing: function contextMenuPopupShowing(aEvent) {
		try {
			var menuSep    = window.document.getElementById('popomungoMenuSeparator_ContentAreaContextMenu');
			var menu       = window.document.getElementById('popomungoMenu_ContentAreaContextMenu');
			var linkNode   = this._findParentLink(document.popupNode);
			var isBookmark = (linkNode.className == org.mozdev.popomungo.bookmarks.BOOKMARK_CLASSNAME);
			
			var menuVisible = false;
			menuVisible = this._enableAddLinkBookmark(!isBookmark, linkNode) || menuVisible;
			menuVisible = this._enableAddPageBookmark(!isBookmark) || menuVisible;
			menuVisible = this._enableDeleteBookmark(isBookmark) || menuVisible;
			menuVisible = this._enableRenameBookmark(isBookmark) || menuVisible;
			
			menuSep.hidden = !menuVisible;
			menu.hidden = !menuVisible;
		}
		catch (err) {
			this._logger.logError(err);
		}
		
		return true;
	},
	

	_enableAddLinkBookmark: function _enableAddLinkBookmark(aEnabled, aLinkNode) {
		try {
			//this._logger.debug("tagName="+ aLinkNode.tagName);
			var menuItem = window.document.getElementById('popomungoMenuItem_AddLinkToBookmarks');
			
			if (aEnabled &&
				aLinkNode.tagName == 'A')
			{
				var hostname = aLinkNode.hostname;
		
				//this._logger.debug("link hostname="+ hostname);
				
				if (hostname.match(org.mozdev.popomungo.commonConstants.POPMUNDO_DOMAIN_REGEXP)) {
					menuItem.hidden = false;
					return true;
				}
			}

			//this._logger.debug("popupNode ignored");
			menuItem.hidden = true;
		}
		catch (err) {
			this._logger.logError(err);
		}
		
		return false;
	},

	
	_enableAddPageBookmark: function _enableAddPageBookmark(aEnabled) {
		try {
			var menuItem = window.document.getElementById('popomungoMenuItem_AddPageToBookmarks');
			
			if (aEnabled) {
				var doc = document.popupNode.ownerDocument;
				
				if (doc) {
					var hostname = doc.location.hostname;
					//this._logger.debug("document hostname="+ hostname);
					
					if (hostname.match(org.mozdev.popomungo.commonConstants.POPMUNDO_DOMAIN_REGEXP)) {
						menuItem.hidden = false;
						return true;
					}
				}
			}

			menuItem.hidden = true;
		}
		catch (err) {
			this._logger.logError(err);
		}
		
		return false;
	},
	
	
	_enableDeleteBookmark: function _enableDeleteBookmark(aEnable) {
		try {
			var menuItem = window.document.getElementById('popomungoMenuItem_DeleteBookmark');
			menuItem.hidden = !aEnable;
			return aEnable;
		}
		catch (err) {
			this._logger.logError(err);
		}
		
		return false;
	},
	

	_enableRenameBookmark: function _enableRenameBookmark(aEnable) {
		try {
			var menuItem = window.document.getElementById('popomungoMenuItem_RenameBookmark');
			menuItem.hidden = !aEnable;
			return aEnable;
		}
		catch (err) {
			this._logger.logError(err);
		}
		
		return false;
	},
	

	_findParentLink: function _findParentLink(aNode) {
		try {
			var node = aNode;
			
			try {
				while (node.tagName != 'A') {
					node = node.parentNode;
				}
				
				return node;
			}
			catch (err) {
				// ignore
			}
			
			return aNode;
		}
		catch (err) {
			this._logger.logError(err);
		}
		
		return null;
	},
	
	
	onCommand_AddPageToBookmarks: function onCommand_AddPageToBookmarks(aDocument) {
		try {
			this._logger.debug("Add page to bookmarks");
			var ownerDoc = aDocument.popupNode.ownerDocument;
			
			var title = org.mozdev.popomungo.commonFunctions.trim(ownerDoc.title);
			
			if (title.indexOf(' - ') > -1) {
				title = title.replace('Popmundo - ', '');
			}
			
			org.mozdev.popomungo.bookmarks.saveBookmark(ownerDoc, ownerDoc.location, title);
			//this._logger.debug("doc="+ aDocument +", ownerDoc="+ ownerDoc);
		}
		catch (err) {
			this._logger.logError(err);
		}
	},
	

	onCommand_AddLinkToBookmarks: function onCommand_AddLinkToBookmarks(aDocument) {
		try {
			this._logger.debug("Add link to bookmarks");
			var ownerDoc = aDocument.popupNode.ownerDocument;
			var node = this._findParentLink(aDocument.popupNode);
			
			var name = org.mozdev.popomungo.commonFunctions.trim(aDocument.popupNode.textContent);
			
			if (name.length == 0) {
				if (node.title) {
					name = org.mozdev.popomungo.commonFunctions.trim(node.title);
				}
				else if (node.alt) {
					name = org.mozdev.popomungo.commonFunctions.trim(node.alt);
				}
				else {
					name = node.href;
				}
			}
			
			org.mozdev.popomungo.bookmarks.saveBookmark(ownerDoc, node, name);
			//this._logger.debug("doc="+ aDocument +", ownerDoc="+ ownerDoc);
		}
		catch (err) {
			this._logger.logError(err);
		}
	},

	
	onCommand_DeleteBookmark: function onCommand_DeleteBookmark(aDocument) {
		try {
			this._logger.debug("Delete bookmark");
			var ownerDoc = aDocument.popupNode.ownerDocument;
			var node = this._findParentLink(aDocument.popupNode);
			org.mozdev.popomungo.bookmarks.deleteBookmark(ownerDoc, node);
		}
		catch (err) {
			this._logger.logError(err);
		}
	},

	
	onCommand_RenameBookmark: function onCommand_RenameBookmark(aDocument) {
		try {
			this._logger.debug("Rename bookmark");
			var ownerDoc = aDocument.popupNode.ownerDocument;
			var node = this._findParentLink(aDocument.popupNode);
			org.mozdev.popomungo.bookmarks.renameBookmark(ownerDoc, node);
		}
		catch (err) {
			this._logger.logError(err);
		}
	},

	
	setContextMenuListener: function setContextMenuListener(aEnabled) {
		try {
			var menu = window.document.getElementById("contentAreaContextMenu");
			
			if (aEnabled && !this.contextMenuListenerEnabled) {
				this.contextMenuListenerEnabled = true;
				this._logger.debug("Add popupshowing listener");
				menu.addEventListener("popupshowing", this, false);
			}
			else if (!aEnabled && this.contextMenuListenerEnabled){
				this.contextMenuListenerEnabled = false;
				this._logger.debug("Remove popupshowing listener");
				menu.removeEventListener("popupshowing", this, false);
			}
			else {
				this._logger.debug('popupshowing listener enabled='+ this.contextMenuListenerEnabled);
			}
		}
		catch (err) {
			this._logger.logError(err);
		}
	}

};

org.mozdev.popomungo.MenuHandler = new org.mozdev.popomungo.classes.MenuHandler();

// EOF